#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Generating Transport Key Containers Using the GOST Algorithms
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9548.txt
#

from pyasn1_alt_modules import rfc9215


# Import from RFC 9215

GostR3410_2012_PublicKey = rfc9215.GostR3410_2012_PublicKey


# The Algorithm Identifier Map update was done by importing rfc9215.
